/*
 * (c) Copyright 2009-2014 PaperCut Software International Pty. Ltd.
 */
package biz.papercut.pcng.ext.paymentgateway;

import java.util.Map;

import biz.papercut.pcng.plugin.TopupAccountOnDemandPlugin;
import biz.papercut.pcng.plugin.TopupAccountOnDemandPlugin2;
import biz.papercut.pcng.plugin.TopupAccountOnDemandPlugin3;
import biz.papercut.pcng.plugin.TopupAccountOnDemandPlugin.AccountType;

/**
 * STUB IMPLEMENTATION for Top-up on demand payment gateways
 * Various helper methods that can be shared between implementations.
 */
public final class GatewayUtils {
     /**
     * Used to pass a reference to gateway-specific code that performs a debit.
     */
    public interface TopUpDebitFunction {
        /**
         * Performs the debit on the top-up gateway.
         *
         * @param amount The amount to debit.
         * @return The amount that was actually transacted, or {@code null} if no transaction took place.
         */
        Double performDebit(double amount);
    }

    /**
     * Helper to perform top-up on demand and retry for a lesser amount if the first didn't work.
     * 
     * This is sufficiently complicated and repeated in each topup-on-demand plugin, so it's been extracted here.
     * 
     * @param plugin The top-up on demand plugin (used to query the available credit if first transfer doesn't work).
     * @param debitFunc A function to perform the debit of the requested amount (may be called more than once)
     * @param topupAmount The topupAmount to try deduct from the external source.
     * @param amountRequired The amount *required* for the print job (i.e. the job cost, which may be less than the top up amount).
     * @param username The user.
     * @param sharedAccount The shared account (or null if personal).
     * @param accountCharged The account charged (shared/personal)
     * @param usageDetails The job usage details.
     * @param txnComment The transaction comment we'll record if the topup is successful. Putting an external transation reference here is highly recommended.
     * @return The topup response (null if no topup performed).
     */
    public static TopupAccountOnDemandPlugin2.TopupResponse performTopupDebit(TopupAccountOnDemandPlugin3 plugin,
            TopUpDebitFunction debitFunc, double topupAmount, double amountRequired, String username,
            String sharedAccount, AccountType accountCharged, Map<String, String> usageDetails, String txnComment) {
        System.out.println("Performing topup on demand for: " + plugin.getClass().getSimpleName());

        // First try transferring the requested amount
        {
            System.out.println("performTopupDebit stub: topupAmount=" + topupAmount
                    + ", amountRequired=" + amountRequired
                    + ", username=" + username
                    + ", sharedAccount=" + sharedAccount
                    + ", accountCharged=" + accountCharged
                    + ", txnComment=" + txnComment);
            
            Double transactionAmount = debitFunc.performDebit(topupAmount);
            if (transactionAmount != null) {
                // Success
                return new TopupAccountOnDemandPlugin.TopupResponse(transactionAmount, txnComment);
            }
        }
        
        // Real implementation will attempt to transfer an amount between amountRequired and topupAmount

        // no top-up was performed
        return null;
    }
    
    /**
     * Parse double amounts, taking into account European localized formats with comma as a decimal place and possible
     * thousands separators.
     *
     * @param value The input string value.
     * @return The parsed double - will be null if input string empty.
     * @throws NumberFormatException if input value cannot be parsed.
     */
    public static Double parseDoubleExtended(String value) throws NumberFormatException {
        String valstr = value.replace(",", ".");
        int dotpos = value.lastIndexOf('.');
        if (dotpos > 4) {
            String lhs = valstr.substring(0, dotpos);
            String rhs = valstr.substring(dotpos);
            
            // Remove any 1,000 separator's
            lhs = lhs.replace(".", "");
            lhs = lhs.replace(" ", "");
            valstr = lhs + rhs;
        }

        Double val = null;
        if (valstr.length() != 0) {
            val = Double.parseDouble(valstr);
        }

        return val;
    }

    /**
     * Private constructor.
     */
    private GatewayUtils() {
        // private constructor
    }
}
